@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ from parsec.upgrade import upgrader, converter
 from cylc import LOG
 from cylc.cfgvalidate import (
     cylc_config_validate, CylcConfigValidator as VDR, DurationFloat)
-from cylc.hostuserutil import is_remote_user
+from cylc.hostuserutil import get_user_home, is_remote_user
 from cylc.mkdir_p import mkdir_p
 from cylc.network import PRIVILEGE_LEVELS, PRIV_STATE_TOTALS, PRIV_SHUTDOWN
 from cylc.version import CYLC_VERSION
@@ -73,6 +73,7 @@ SPEC = {
             'timeout handler': [VDR.V_STRING_LIST],
             'inactivity handler': [VDR.V_STRING_LIST],
             'shutdown handler': [VDR.V_STRING_LIST],
+            'aborted handler': [VDR.V_STRING_LIST],
             'stalled handler': [VDR.V_STRING_LIST],
             'timeout': [VDR.V_INTERVAL],
             'inactivity': [VDR.V_INTERVAL],
@@ -89,19 +90,11 @@ SPEC = {
 
     'documentation': {
         'files': {
-            'html index': [
-                VDR.V_STRING, '$CYLC_DIR/doc/install/index.html'],
-            'pdf user guide': [
-                VDR.V_STRING, '$CYLC_DIR/doc/install/cylc-user-guide.pdf'],
-            'multi-page html user guide': [
-                VDR.V_STRING,
-                '$CYLC_DIR/doc/install/html/multi/cug-html.html'],
-            'single-page html user guide': [
-                VDR.V_STRING,
-                '$CYLC_DIR/doc/install/html/single/cug-html.html'],
+            'html user guides': [
+                VDR.V_STRING, '$CYLC_DIR/doc/built-sphinx/index.html'],
         },
         'urls': {
-            'internet homepage': [VDR.V_STRING, 'http://cylc.github.io/cylc/'],
+            'internet homepage': [VDR.V_STRING, 'http://cylc.github.io/'],
             'local index': [VDR.V_STRING],
         },
     },
@@ -205,7 +198,6 @@ SPEC = {
         'mail retry delays': [VDR.V_INTERVAL_LIST],
         'mail smtp': [VDR.V_STRING],
         'mail to': [VDR.V_STRING],
-        'reset timer': [VDR.V_BOOLEAN],
         'submission timeout': [VDR.V_INTERVAL],
     },
 
@@ -257,40 +249,7 @@ SPEC = {
 
 def upg(cfg, descr):
     """Upgrader."""
-    add_bin_dir = converter(lambda x: x + '/bin', "Added + '/bin' to path")
-    use_ssh = converter(lambda x: "ssh", "set to 'ssh'")
-
     u = upgrader(cfg, descr)
-    u.deprecate(
-        '5.1.1',
-        ['editors', 'in-terminal'],
-        ['editors', 'terminal'])
-    u.deprecate(
-        '5.1.1',
-        ['task hosts'],
-        ['hosts'])
-    u.deprecate(
-        '5.1.1',
-        ['hosts', 'local'],
-        ['hosts', 'localhost'])
-    u.deprecate(
-        '5.1.1',
-        ['hosts', '__MANY__', 'workspace directory'],
-        ['hosts', '__MANY__', 'workdirectory'])
-    u.deprecate(
-        '5.1.1',
-        ['hosts', '__MANY__', 'cylc directory'],
-        ['hosts', '__MANY__', 'cylc bin directory'],
-        add_bin_dir)
-    u.obsolete(
-        '5.2.0',
-        ['hosts', '__MANY__', 'cylc bin directory'],
-        ['hosts', '__MANY__', 'cylc bin directory'])
-    u.deprecate(
-        '5.2.0',
-        ['hosts', '__MANY__', 'use ssh messaging'],
-        ['hosts', '__MANY__', 'task communication method'],
-        use_ssh)
     u.deprecate(
         '6.1.2',
         ['task messaging', 'connection timeout in seconds'],
@@ -365,8 +324,7 @@ def upg(cfg, descr):
     # A special remote tail command template is no longer needed. It used to
     # use 'tail -pid' to kill 'tail' on ssh exit, but we do this in cylc now.
     u.obsolete(
-        '7.6.0',
-        ['hosts', '__MANY__', 'remote tail command template'])
+        '7.6.0', ['hosts', '__MANY__', 'remote tail command template'])
     u.deprecate(
         '7.6.0',
         ['hosts', '__MANY__', 'local tail command template'],
@@ -380,20 +338,14 @@ def upg(cfg, descr):
             int(x) + int(cfg['communication']['maximum number of ports'])),
             "Format as range list"))
     u.obsolete(
-        '7.8.0',
-        ['communication', 'maximum number of ports'])
+        '7.8.0', ['communication', 'maximum number of ports'])
     u.deprecate(
-        '7.8.0',
-        ['suite host scanning'],
-        ['suite servers'])
+        '7.8.0', ['suite host scanning'], ['suite servers'])
     u.deprecate(
-        '7.8.0',
-        ['suite servers', 'hosts'],
-        ['suite servers', 'scan hosts'])
+        '7.8.0', ['suite servers', 'hosts'], ['suite servers', 'scan hosts'])
     # Roll over is always done.
-    u.obsolete(
-        '7.8.0',
-        ['suite logging', 'roll over at start-up'])
+    u.obsolete('7.8.0', ['suite logging', 'roll over at start-up'])
+    u.obsolete('7.8.1', ['cylc', 'events', 'reset timer'])
     u.upgrade()
 
 
@@ -414,6 +366,7 @@ class GlobalConfig(ParsecConfig):
     """
 
     _DEFAULT = None
+    _HOME = os.getenv('HOME') or get_user_home()
     # For working with git clones, just go to minor version number.
     cylc_version = re.sub('-.*', '', CYLC_VERSION)
     CONF_BASE = "global.rc"
@@ -421,8 +374,8 @@ class GlobalConfig(ParsecConfig):
     SITE_CONF_DIR = os.path.join(os.environ["CYLC_DIR"], "etc")
     SITE_CONF_DIR_OLD = os.path.join(os.environ["CYLC_DIR"], "conf")
     # User global.rc loc preference: if not in .cylc/x.y.z/ look in .cylc/.
-    USER_CONF_DIR_1 = os.path.join(os.environ['HOME'], '.cylc', cylc_version)
-    USER_CONF_DIR_2 = os.path.join(os.environ['HOME'], '.cylc')
+    USER_CONF_DIR_1 = os.path.join(_HOME, '.cylc', cylc_version)
+    USER_CONF_DIR_2 = os.path.join(_HOME, '.cylc')
 
     @classmethod
     def get_inst(cls, cached=True):
@@ -564,13 +517,13 @@ class GlobalConfig(ParsecConfig):
         if value is not None and 'directory' in item:
             if replace_home or modify_dirs:
                 # Replace local home dir with $HOME for eval'n on other host.
-                value = value.replace(os.environ['HOME'], '$HOME')
+                value = value.replace(self._HOME, '$HOME')
             elif is_remote_user(owner):
                 # Replace with ~owner for direct access via local filesys
                 # (works for standard cylc-run directory location).
                 if owner_home is None:
                     owner_home = os.path.expanduser('~%s' % owner)
-                value = value.replace(os.environ['HOME'], owner_home)
+                value = value.replace(self._HOME, owner_home)
         if item == "task communication method" and value == "default":
             # Translate "default" to client-server comms: "https" or "http".
             value = cfg['communication']['method']
@@ -659,6 +612,8 @@ class GlobalConfig(ParsecConfig):
 
         Host item values of None default to modified localhost values.
         Expand environment variables and ~ notations.
+
+        Ensure os.environ['HOME'] is defined with the correct value.
         """
         cfg = self.get()
 
@@ -673,13 +628,14 @@ class GlobalConfig(ParsecConfig):
                 if newvalue and 'directory' in item:
                     # replace local home dir with $HOME for evaluation on other
                     # host
-                    newvalue = newvalue.replace(os.environ['HOME'], '$HOME')
+                    newvalue = newvalue.replace(self._HOME, '$HOME')
                 cfg['hosts'][host][item] = newvalue
 
         # Expand environment variables and ~user in LOCAL file paths.
+        if 'HOME' not in os.environ:
+            os.environ['HOME'] = self._HOME
         for key, val in cfg['documentation']['files'].items():
             cfg['documentation']['files'][key] = os.path.expandvars(val)
-
         for key, val in cfg['hosts']['localhost'].items():
             if val and 'directory' in key:
                 cfg['hosts']['localhost'][key] = os.path.expandvars(val)
