@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -74,11 +74,10 @@ SPEC = {
             'timeout handler': [VDR.V_STRING_LIST, None],
             'inactivity handler': [VDR.V_STRING_LIST, None],
             'shutdown handler': [VDR.V_STRING_LIST, None],
+            'aborted handler': [VDR.V_STRING_LIST, None],
             'stalled handler': [VDR.V_STRING_LIST, None],
             'timeout': [VDR.V_INTERVAL],
             'inactivity': [VDR.V_INTERVAL],
-            'reset timer': [VDR.V_BOOLEAN, True],
-            'reset inactivity timer': [VDR.V_BOOLEAN, True],
             'abort if startup handler fails': [VDR.V_BOOLEAN],
             'abort if shutdown handler fails': [VDR.V_BOOLEAN],
             'abort if timeout handler fails': [VDR.V_BOOLEAN],
@@ -217,7 +216,6 @@ SPEC = {
                 'mail retry delays': [VDR.V_INTERVAL_LIST, None],
                 'mail smtp': [VDR.V_STRING],
                 'mail to': [VDR.V_STRING],
-                'reset timer': [VDR.V_BOOLEAN],
                 'submission timeout': [VDR.V_INTERVAL],
 
                 'expired handler': [VDR.V_STRING_LIST, None],
@@ -381,6 +379,9 @@ def upg(cfg, descr):
     u.obsolete('7.6.0', ['runtime', '__MANY__', 'enable resurrection'])
     u.obsolete('7.8.0', ['runtime', '__MANY__', 'suite state polling',
                          'template'])
+    u.obsolete('7.8.1', ['cylc', 'events', 'reset timer'])
+    u.obsolete('7.8.1', ['cylc', 'events', 'reset inactivity timer'])
+    u.obsolete('7.8.1', ['runtime', '__MANY__', 'events', 'reset timer'])
     u.upgrade()
 
 
