@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Parse and validate the suite definition file
 
+Set local values of variables to give suite context before parsing
+config, i.e for template filters (Jinja2, python ...etc) and possibly
+needed locally by event handlers. This is needed for both running and
+non-running suite parsing (obtaining config/graph info). Potentially
+task-specific due to different directory paths on different task hosts,
+however, they are overridden by tasks prior to job submission.
+
 Do some consistency checking, then construct task proxy objects and graph
 structures.
 """
@@ -46,16 +53,18 @@ from cylc.cycling.loader import (
     get_sequence, get_sequence_cls, init_cyclers, INTEGER_CYCLING_TYPE,
     ISO8601_CYCLING_TYPE)
 from cylc.cycling import IntervalParsingError
+from cylc.cycling.iso8601 import ingest_time
 import cylc.flags
 from cylc.graphnode import GraphNodeParser, GraphNodeError
 from cylc.print_tree import print_tree
 from cylc.subprocctx import SubFuncContext
+from cylc.subprocpool import get_func
 from cylc.suite_srv_files_mgr import SuiteSrvFilesManager
 from cylc.taskdef import TaskDef, TaskDefError
 from cylc.task_id import TaskID
 from cylc.task_outputs import TASK_OUTPUT_SUCCEEDED
 from cylc.task_trigger import TaskTrigger, Dependency
-from cylc.wallclock import get_current_time_string, set_utc_mode
+from cylc.wallclock import get_current_time_string, set_utc_mode, get_utc_mode
 from cylc.xtrigger_mgr import XtriggerManager
 
 
@@ -82,16 +91,9 @@ def check_varnames(env):
 
 
 class SuiteConfigError(Exception):
-    """
-    Attributes:
-        message - what the problem is.
-        TODO - element - config element causing the problem
-    """
-    def __init__(self, msg):
-        self.msg = msg
+    """Catch all exception for suite configuration errors."""
+    pass
 
-    def __str__(self):
-        return repr(self.msg)
 
 # TODO: separate config for run and non-run purposes?
 
@@ -101,17 +103,25 @@ class SuiteConfig(object):
 
     Q_DEFAULT = 'default'
     TASK_EVENT_TMPL_KEYS = (
-        'event', 'suite', 'point', 'name', 'submit_num', 'id', 'message',
-        'batch_sys_name', 'batch_sys_job_id', 'submit_time', 'start_time',
-        'finish_time', 'user@host', 'try_num')
+        'event', 'suite', 'suite_uuid', 'point', 'name', 'submit_num', 'id',
+        'message', 'batch_sys_name', 'batch_sys_job_id', 'submit_time',
+        'start_time', 'finish_time', 'user@host', 'try_num')
 
-    def __init__(self, suite, fpath, template_vars=None,
-                 owner=None, run_mode='live', is_validate=False, strict=False,
-                 collapsed=None, cli_initial_point_string=None,
-                 cli_start_point_string=None, cli_final_point_string=None,
-                 is_reload=False, output_fname=None,
-                 vis_start_string=None, vis_stop_string=None,
-                 xtrigger_mgr=None, mem_log_func=None):
+    def __init__(
+        self,
+        suite,
+        fpath,
+        options=None,
+        template_vars=None,
+        is_reload=False,
+        output_fname=None,
+        xtrigger_mgr=None,
+        mem_log_func=None,
+        run_dir=None,
+        log_dir=None,
+        work_dir=None,
+        share_dir=None,
+    ):
 
         self.mem_log = mem_log_func
         if mem_log_func is None:
@@ -120,14 +130,21 @@ class SuiteConfig(object):
         self.suite = suite  # suite name
         self.fpath = fpath  # suite definition
         self.fdir = os.path.dirname(fpath)
-        self.owner = owner
-        self.run_mode = run_mode
-        self.strict = strict
+        self.run_dir = run_dir or glbl_cfg().get_derived_host_item(
+            self.suite, 'suite run directory')
+        self.log_dir = log_dir or glbl_cfg().get_derived_host_item(
+            self.suite, 'suite log directory')
+        self.work_dir = work_dir or glbl_cfg().get_derived_host_item(
+            self.suite, 'suite work directory')
+        self.share_dir = share_dir or glbl_cfg().get_derived_host_item(
+            self.suite, 'suite share directory')
+        self.options = options
         self.naked_dummy_tasks = []
         self.edges = {}
         self.taskdefs = {}
         self.initial_point = None
         self.start_point = None
+        self.final_point = None
         self.first_graph = True
         self.clock_offsets = {}
         self.expiration_offsets = {}
@@ -135,13 +152,11 @@ class SuiteConfig(object):
         self.ext_triggers = {}
         if xtrigger_mgr is None:
             # For validation and graph etc.
-            self.xtrigger_mgr = XtriggerManager(self.suite, self.owner)
+            self.xtrigger_mgr = XtriggerManager(self.suite)
         else:
             self.xtrigger_mgr = xtrigger_mgr
         self.xtriggers = {}
         self.suite_polling_tasks = {}
-        self.vis_start_point_string = vis_start_string
-        self.vis_stop_point_string = vis_stop_string
         self._last_graph_raw_id = None
         self._last_graph_raw_edges = []
 
@@ -174,6 +189,9 @@ class SuiteConfig(object):
         # one up from root
         self.feet = []
 
+        # Export local environmental suite context before config parsing.
+        self.process_suite_env()
+
         # parse, upgrade, validate the suite, but don't expand with default
         # items
         self.mem_log("config.py: before RawSuiteConfig init")
@@ -199,9 +217,9 @@ class SuiteConfig(object):
                 "'max active cycle points', not both")
 
         # Override the suite defn with an initial point from the CLI.
-        if cli_initial_point_string is not None:
-            self.cfg['scheduling']['initial cycle point'] = (
-                cli_initial_point_string)
+        icp_str = getattr(self.options, 'icp', None)
+        if icp_str is not None:
+            self.cfg['scheduling']['initial cycle point'] = icp_str
 
         dependency_map = self.cfg.get('scheduling', {}).get(
             'dependencies', {})
@@ -227,10 +245,15 @@ class SuiteConfig(object):
                 fcp = self.cfg['scheduling'].get('final cycle point')
                 if just_has_async_graph and not (
                         icp in [None, "1"] and fcp in [None, icp]):
-                    raise SuiteConfigError(
-                        'Conflicting syntax: integer vs ' +
-                        'cycling suite: ' +
-                        'are you missing a [dependencies][[[R1]]] section?')
+                    msg = (
+                        'Conflicting syntax: integer vs cycling suite:'
+                        '\n* Are you missing a [dependencies][R1] section?')
+                    if icp_str:
+                        msg += (
+                            '\n* Did you mean to provide "%s" as the initial '
+                            'cycle point?' % icp_str)
+                    raise SuiteConfigError(msg)
+
                 if just_has_async_graph:
                     # There aren't any other graphs, so set integer cycling.
                     self.cfg['scheduling']['cycling mode'] = (
@@ -346,14 +369,19 @@ class SuiteConfig(object):
                 "This suite requires an initial cycle point.")
         if icp == "now":
             icp = get_current_time_string()
+        else:
+            try:
+                my_now = get_current_time_string()
+                icp = ingest_time(icp, my_now)
+            except ValueError as exc:
+                raise SuiteConfigError(str(exc))
         self.initial_point = get_point(icp).standardise()
         self.cfg['scheduling']['initial cycle point'] = str(self.initial_point)
-        if cli_start_point_string:
+        if getattr(self.options, 'startcp', None) is not None:
             # Warm start from a point later than initial point.
-            if cli_start_point_string == "now":
-                cli_start_point_string = get_current_time_string()
-            cli_start_point = get_point(cli_start_point_string).standardise()
-            self.start_point = cli_start_point
+            if self.options.startcp == "now":
+                self.options.startcp = get_current_time_string()
+            self.start_point = get_point(self.options.startcp).standardise()
         else:
             # Cold start.
             self.start_point = self.initial_point
@@ -374,55 +402,53 @@ class SuiteConfig(object):
                     self.cfg['scheduling']['initial cycle point constraints'])
                 raise SuiteConfigError(
                     ("Initial cycle point %s does not meet the constraints " +
-                     "%s") % (
-                        str(self.initial_point),
-                        constraints_str
-                    )
+                     "%s") % (str(self.initial_point), constraints_str)
                 )
 
         if (self.cfg['scheduling']['final cycle point'] is not None and
                 self.cfg['scheduling']['final cycle point'].strip() is ""):
             self.cfg['scheduling']['final cycle point'] = None
-        final_point_string = (cli_final_point_string or
-                              self.cfg['scheduling']['final cycle point'])
-        final_point = None
-        if final_point_string is not None:
+        fcp_str = getattr(self.options, 'fcp', None)
+        if fcp_str is None:
+            fcp_str = self.cfg['scheduling']['final cycle point']
+        if fcp_str is not None:
             # Is the final "point"(/interval) relative to initial?
             if get_interval_cls().get_null().TYPE == INTEGER_CYCLING_TYPE:
-                if "P" in final_point_string:
+                if "P" in fcp_str:
                     # Relative, integer cycling.
-                    final_point = get_point_relative(
+                    self.final_point = get_point_relative(
                         self.cfg['scheduling']['final cycle point'],
                         self.initial_point
                     ).standardise()
             else:
                 try:
                     # Relative, ISO8601 cycling.
-                    final_point = get_point_relative(
-                        final_point_string, self.initial_point).standardise()
+                    self.final_point = get_point_relative(
+                        fcp_str, self.initial_point).standardise()
                 except ValueError:
                     # (not relative)
                     pass
-            if final_point is None:
+            if self.final_point is None:
                 # Must be absolute.
-                final_point = get_point(final_point_string).standardise()
-            self.cfg['scheduling']['final cycle point'] = str(final_point)
+                self.final_point = get_point(fcp_str).standardise()
+            self.cfg['scheduling']['final cycle point'] = str(self.final_point)
 
-        if final_point is not None and self.initial_point > final_point:
+        if (self.final_point is not None and
+                self.initial_point > self.final_point):
             raise SuiteConfigError(
                 "The initial cycle point:" +
                 str(self.initial_point) + " is after the final cycle point:" +
-                str(final_point) + ".")
+                str(self.final_point) + ".")
 
         # Validate final cycle point against any constraints
-        if (final_point is not None and
+        if (self.final_point is not None and
                 self.cfg['scheduling']['final cycle point constraints']):
             valid_fcp = False
             for entry in (
                     self.cfg['scheduling']['final cycle point constraints']):
                 possible_pt = get_point_relative(
-                    entry, final_point).standardise()
-                if final_point == possible_pt:
+                    entry, self.final_point).standardise()
+                if self.final_point == possible_pt:
                     valid_fcp = True
                     break
             if not valid_fcp:
@@ -430,7 +456,7 @@ class SuiteConfig(object):
                     self.cfg['scheduling']['final cycle point constraints'])
                 raise SuiteConfigError(
                     "Final cycle point %s does not meet the constraints %s" % (
-                        str(final_point), constraints_str))
+                        str(self.final_point), constraints_str))
 
         # Parse special task cycle point offsets, and replace family names.
         LOG.debug("Parsing [special tasks]")
@@ -509,10 +535,9 @@ class SuiteConfig(object):
                     'ERROR [visualization]collapsed families: '
                     '%s is not a first parent' % fam)
 
-        if is_reload and collapsed:
-            # on suite reload retain an existing state of collapse
+        if getattr(options, 'collapsed', None):
             # (used by the "cylc graph" viewer)
-            self.closed_families = collapsed
+            self.closed_families = getattr(self.options, 'collapsed', None)
         elif is_reload:
             self.closed_families = []
         else:
@@ -525,15 +550,11 @@ class SuiteConfig(object):
                         '[visualization][collapsed families]: ' +
                         'family ' + cfam + ' not defined')
 
-        # check for run mode override at suite level
-        if self.cfg['cylc']['force run mode']:
-            self.run_mode = self.cfg['cylc']['force run mode']
-
-        self.process_directories()
+        self.process_config_env()
 
         self.mem_log("config.py: before load_graph()")
         self.load_graph()
-        if not is_validate:
+        if self._is_validate():
             GraphNodeParser.get_inst().clear()
         self.mem_log("config.py: after load_graph()")
 
@@ -541,7 +562,7 @@ class SuiteConfig(object):
 
         self.configure_queues()
 
-        if self.run_mode in ['simulation', 'dummy', 'dummy-local']:
+        if self.run_mode('simulation', 'dummy', 'dummy-local'):
             self.configure_sim_modes()
 
         self.configure_suite_state_polling_tasks()
@@ -549,17 +570,17 @@ class SuiteConfig(object):
         # Warn or abort (if --strict) if naked dummy tasks (no runtime
         # section) are found in graph or queue config.
         if len(self.naked_dummy_tasks) > 0:
-            if self.strict or cylc.flags.verbose:
+            if self._is_validate(is_strict=True) or cylc.flags.verbose:
                 err_msg = ('naked dummy tasks detected (no entry'
                            ' under [runtime]):')
                 for ndt in self.naked_dummy_tasks:
                     err_msg += '\n+\t' + str(ndt)
                 LOG.warning(err_msg)
-            if self.strict:
+            if self._is_validate(is_strict=True):
                 raise SuiteConfigError(
                     'ERROR: strict validation fails naked dummy tasks')
 
-        if is_validate:
+        if self._is_validate():
             self.check_tasks()
 
         # Check that external trigger messages are only used once (they have to
@@ -641,14 +662,6 @@ class SuiteConfig(object):
             raise SuiteConfigError("Node attributes must be of the form "
                                    "'key1=value1', 'key2=value2', etc.")
 
-        # (Note that we're retaining 'default node attributes' even
-        # though this could now be achieved by styling the root family,
-        # because putting default attributes for root in the suite.rc spec
-        # results in root appearing last in the ordered dict of node
-        # names, so it overrides the styling for lesser groups and
-        # nodes, whereas the reverse is needed - fixing this would
-        # require reordering task_attr in lib/cylc/graphing.py).
-
         self.leaves = self.get_task_name_list()
         for ancestors in self.runtime['first-parent ancestors'].values():
             try:
@@ -660,12 +673,10 @@ class SuiteConfig(object):
                     self.feet.append(foot)
 
         # CLI override for visualization settings.
-        if self.vis_start_point_string:
-            self.cfg['visualization']['initial cycle point'] = (
-                self.vis_start_point_string)
-        if self.vis_stop_point_string:
-            self.cfg['visualization']['final cycle point'] = (
-                self.vis_stop_point_string)
+        for key in ('initial', 'final'):
+            vis_str = getattr(self.options, 'vis_' + key, None)
+            if vis_str:
+                self.cfg['visualization'][key + ' cycle point'] = vis_str
 
         # For static visualization, start point defaults to suite initial
         # point; stop point must be explicit with initial point, or None.
@@ -694,10 +705,10 @@ class SuiteConfig(object):
             vfcp = None
 
         # A viz final point can't be beyond the suite final point.
-        if vfcp is not None and final_point is not None:
-            if vfcp > final_point:
+        if vfcp is not None and self.final_point is not None:
+            if vfcp > self.final_point:
                 self.cfg['visualization']['final cycle point'] = str(
-                    final_point)
+                    self.final_point)
 
         # Replace suite and task name in suite and task URLs.
         self.cfg['meta']['URL'] = self.cfg['meta']['URL'] % {
@@ -714,7 +725,7 @@ class SuiteConfig(object):
             cfg['meta']['URL'] = RE_TASK_NAME_VAR.sub(
                 name, cfg['meta']['URL'])
 
-        if is_validate:
+        if self._is_validate():
             self.mem_log("config.py: before _check_circular()")
             self._check_circular()
             self.mem_log("config.py: after _check_circular()")
@@ -728,7 +739,9 @@ class SuiteConfig(object):
         lhs2rhss = {}  # left hand side to right hand sides
         rhs2lhss = {}  # right hand side to left hand sides
         for lhs, rhs in self.get_graph_raw(
-                start_point_string, stop_point_string=None, is_validate=True):
+            start_point_string,
+            stop_point_string=None,
+        ):
             lhs2rhss.setdefault(lhs, set())
             lhs2rhss[lhs].add(rhs)
             rhs2lhss.setdefault(rhs, set())
@@ -858,6 +871,13 @@ class SuiteConfig(object):
                             return True
         return False
 
+    def _is_validate(self, is_strict=False):
+        """Return whether we are in (strict) validate mode."""
+        return (
+            getattr(self.options, 'is_validate', False) and
+            (not is_strict or getattr(self.options, 'strict', False))
+        )
+
     @staticmethod
     def dequote(s):
         """Strip quotes off a string."""
@@ -1260,8 +1280,8 @@ class SuiteConfig(object):
                 ).get_seconds()
             rtc['job']['execution time limit'] = (
                 sleep_sec + DurationParser().parse(str(
-                    rtc['simulation']['time limit buffer'])
-                ).get_seconds())
+                    rtc['simulation']['time limit buffer'])).get_seconds()
+            )
             rtc['job']['simulated run length'] = sleep_sec
 
             # Generate dummy scripting.
@@ -1409,9 +1429,53 @@ class SuiteConfig(object):
 
         print_tree(tree, padding=padding, use_unicode=pretty)
 
-    def process_directories(self):
-        os.environ['CYLC_SUITE_NAME'] = self.suite
-        os.environ['CYLC_SUITE_DEF_PATH'] = self.fdir
+    def process_suite_env(self):
+        """Suite context is exported to the local environment."""
+        for var, val in [
+                ('CYLC_SUITE_NAME', self.suite),
+                ('CYLC_DEBUG', str(cylc.flags.debug).lower()),
+                ('CYLC_VERBOSE', str(cylc.flags.verbose).lower()),
+                ('CYLC_SUITE_DEF_PATH', self.fdir),
+                ('CYLC_SUITE_RUN_DIR', self.run_dir),
+                ('CYLC_SUITE_LOG_DIR', self.log_dir),
+                ('CYLC_SUITE_WORK_DIR', self.work_dir),
+                ('CYLC_SUITE_SHARE_DIR', self.share_dir)]:
+            os.environ[var] = val
+
+    def process_config_env(self):
+        """Set local config derived environment."""
+        os.environ['CYLC_UTC'] = str(get_utc_mode())
+        os.environ['CYLC_SUITE_INITIAL_CYCLE_POINT'] = str(self.initial_point)
+        os.environ['CYLC_SUITE_FINAL_CYCLE_POINT'] = str(self.final_point)
+        os.environ['CYLC_CYCLING_MODE'] = self.cfg['scheduling'][
+            'cycling mode']
+        #     (global config auto expands environment variables in local paths)
+        cenv = self.cfg['cylc']['environment'].copy()
+        for var, val in cenv.items():
+            cenv[var] = os.path.expandvars(val)
+        #     path to suite bin directory for suite and event handlers
+        cenv['PATH'] = os.pathsep.join([
+            os.path.join(self.fdir, 'bin'), os.environ['PATH']])
+        #     and to suite event handlers in this process.
+        for var, val in cenv.items():
+            os.environ[var] = val
+
+    def run_mode(self, *reqmodes):
+        """Return the run mode.
+
+        Combine command line option with configuration setting.
+        If "reqmodes" is specified, return the boolean (mode in reqmodes).
+        Otherwise, return the mode as a str.
+        """
+        mode = getattr(self.options, 'run_mode', None)
+        if not mode:
+            mode = self.cfg['cylc']['force run mode']
+        if not mode:
+            mode = 'live'
+        if reqmodes:
+            return mode in reqmodes
+        else:
+            return mode
 
     def check_tasks(self):
         """Call after all tasks are defined.
@@ -1484,8 +1548,8 @@ class SuiteConfig(object):
                 if (name not in self.taskdefs and
                         name not in self.cfg['runtime']):
                     msg = '%s task "%s" is not defined.' % (task_type, name)
-                    if self.strict:
-                        raise SuiteConfigError("ERROR: " + msg)
+                    if self._is_validate(is_strict=True):
+                        raise SuiteConfigError("ERROR: %s" % msg)
                     else:
                         LOG.warning(msg)
 
@@ -1706,7 +1770,7 @@ class SuiteConfig(object):
     def get_graph_raw(self, start_point_string, stop_point_string,
                       group_nodes=None, ungroup_nodes=None,
                       ungroup_recursive=False, group_all=False,
-                      ungroup_all=False, is_validate=False):
+                      ungroup_all=False):
         """Convert the abstract graph edges (self.edges, etc) to actual edges
 
         Actual edges have concrete ranges of cycle points.
@@ -1714,6 +1778,7 @@ class SuiteConfig(object):
         In validate mode, set ungroup_all to True, and only return non-suicide
         edges with left and right nodes.
         """
+        is_validate = self._is_validate()
         if is_validate:
             ungroup_all = True
         if group_nodes is None:
@@ -2050,6 +2115,17 @@ class SuiteConfig(object):
                         if offset > old_offset:
                             self.taskdefs[task_name].xclock_label = label
                 else:
+                    try:
+                        if not callable(get_func(xtrig.func_name, self.fdir)):
+                            raise SuiteConfigError(
+                                "ERROR, "
+                                "xtrigger function not callable: %s" %
+                                xtrig.func_name)
+                    except (ImportError, AttributeError):
+                        raise SuiteConfigError(
+                            "ERROR, "
+                            "xtrigger function not found: %s" %
+                            xtrig.func_name)
                     self.xtrigger_mgr.add_trig(label, xtrig)
                     self.taskdefs[task_name].xtrig_labels.add(label)
 
@@ -2126,7 +2202,7 @@ class SuiteConfig(object):
 
         # Get the taskdef object for generating the task proxy class
         taskd = TaskDef(
-            name, rtcfg, self.run_mode, self.start_point,
+            name, rtcfg, self.run_mode(), self.start_point,
             self.cfg['scheduling']['spawn to max active cycle points'])
 
         # TODO - put all taskd.foo items in a single config dict
