@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 import glib
 import os
+import pkg_resources
 import sys
 import traceback
 
@@ -97,12 +98,9 @@ class EntryDialog(gtk.MessageDialog):
 
 def get_image_dir():
     """Return the root directory for cylc images."""
-    try:
-        cylc_dir = os.environ['CYLC_DIR']
-    except KeyError:
-        # This should not happen (unnecessary)
-        raise SystemExit("ERROR: $CYLC_DIR is not defined!")
-    return os.path.join(cylc_dir, "images")
+    resource_package = __name__
+    resource_path = "/images/"
+    return pkg_resources.resource_filename(resource_package, resource_path)
 
 
 def get_icon():
