@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -67,9 +67,6 @@ avoid this, use the '--no-multitask-compat' option, or use the new syntax
 
         if multitask:
             usage += self.MULTITASK_USAGE
-        usage += """
-
-Arguments:"""
         args = ""
         self.n_compulsory_args = 0
         self.n_optional_args = 0
@@ -87,20 +84,21 @@ Arguments:"""
             if len(arg[0]) > maxlen:
                 maxlen = len(arg[0])
 
-        for arg in argdoc:
-            if arg[0].startswith('['):
-                self.n_optional_args += 1
-            else:
-                self.n_compulsory_args += 1
-            if arg[0].endswith('...]'):
-                self.unlimited_args = True
+        if argdoc:
+            usage += "\n\nArguments:"
+            for arg in argdoc:
+                if arg[0].startswith('['):
+                    self.n_optional_args += 1
+                else:
+                    self.n_compulsory_args += 1
+                if arg[0].endswith('...]'):
+                    self.unlimited_args = True
 
-            args += arg[0] + " "
+                args += arg[0] + " "
 
-            pad = (maxlen - len(arg[0])) * ' ' + '               '
-            usage += "\n   " + arg[0] + pad + arg[1]
-
-        usage = usage.replace('ARGS', args)
+                pad = (maxlen - len(arg[0])) * ' ' + '               '
+                usage += "\n   " + arg[0] + pad + arg[1]
+            usage = usage.replace('ARGS', args)
 
         OptionParser.__init__(self, usage)
 
@@ -249,11 +247,15 @@ Arguments:"""
 
         if self.icp:
             self.add_option(
-                "--icp",
+                "--initial-cycle-point", "--initial-point", "--icp", "--ict",
                 metavar="CYCLE_POINT",
                 help=(
-                    "Set initial cycle point. "
-                    "Required if not defined in suite.rc."))
+                    "Set the initial cycle point. "
+                    "Required if not defined in suite.rc."
+                ),
+                action="store",
+                dest="icp",
+            )
 
     def parse_args(self, remove_opts=None):
         """Parse options and arguments, overrides OptionParser.parse_args."""
@@ -285,11 +287,20 @@ Arguments:"""
         cylc.flags.verbose = options.verbose
         cylc.flags.debug = options.debug
 
-        # Set up stream logging
+        # Set up stream logging for CLI. Note:
+        # 1. On choosing STDERR: Log messages are diagnostics, so STDERR is the
+        #    better choice for the logging stream. This allows us to use STDOUT
+        #    for verbosity agnostic outputs.
+        # 2. Suite server programs will remove this handler when it becomes a
+        #    daemon.
         if options.debug or options.verbose:
             LOG.setLevel(logging.DEBUG)
         else:
             LOG.setLevel(logging.INFO)
+        # Remove NullHandler before add the StreamHandler
+        while LOG.handlers:
+            LOG.handlers[0].close()
+            LOG.removeHandler(LOG.handlers[0])
         errhandler = logging.StreamHandler(sys.stderr)
         errhandler.setFormatter(CylcLogFormatter())
         LOG.addHandler(errhandler)
