@@ -1,5 +1,5 @@
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,16 +21,18 @@ System calls to cylc are performed here.
 
 import os
 import shutil
-from subprocess import Popen, PIPE, call
 import sys
 import tempfile
 import time
 import traceback
+from subprocess import PIPE, Popen, call  # nosec
 
-from . import (PROFILE_MODE_TIME, PROFILE_MODE_CYLC, PROFILE_MODES,
-               PROFILE_FILES, SUITE_STARTUP_STRING)
+from cylc.cylc_subproc import procopen
+
+from . import (PROFILE_FILES, PROFILE_MODE_CYLC, PROFILE_MODE_TIME,
+               PROFILE_MODES, SUITE_STARTUP_STRING)
 from .analysis import extract_results
-from .git import (checkout, describe, GitCheckoutError,)
+from .git import GitCheckoutError, checkout, describe
 
 
 def cylc_env(cylc_conf_path=''):
@@ -186,9 +188,13 @@ def run_suite(reg, options, out_file, profile_modes, mode='live',
 
     # Execute.
     print '$ ' + ' '.join(cmds)
+
     try:
-        proc = Popen(' '.join(cmds), shell=True, stderr=open(time_err, 'w+'),
-                     stdout=open(startup_file, 'w+'), env=env)
+        proc = procopen([' '.join(cmds)], usesh=True,
+                        stderr=open(time_err, 'w+'),
+                        # calls to open a shell are aggregated in
+                        # cylc_subproc.procopen()
+                        stdout=open(startup_file, 'w+'), env=env)
         if proc.wait():
             raise SuiteFailedException(run_cmds, cmd_out, cmd_err)
     except KeyboardInterrupt:
