@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -897,8 +897,8 @@ class CylcSuiteDAO(object):
             return
         for name, key in [
                 ("run mode", "run_mode"),
-                ("initial cycle", "initial_point"),
-                ("final cycle", "final_point")]:
+                ("initial cycle", "icp"),
+                ("final cycle", "fcp")]:
             if tail == "None":
                 tail = None
             if head == name:
