@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -198,10 +198,19 @@ To start a new run, stop the old one first with one or more of these:
 
     def dump_contact_file(self, reg, data):
         """Create contact file. Data should be a key=value dict."""
+        # Note:
+        # 1st fsync for writing the content of the contact file to disk.
+        # 2nd fsync for writing the file metadata of the contact file to disk.
+        # The double fsync logic ensures that if the contact file is written to
+        # a shared file system e.g. via NFS, it will be immediately visible
+        # from by a process on other hosts after the current process returns.
         with open(self.get_contact_file(reg), "wb") as handle:
             for key, value in sorted(data.items()):
                 handle.write("%s=%s\n" % (key, value))
             os.fsync(handle.fileno())
+        dir_fileno = os.open(self.get_suite_srv_dir(reg), os.O_DIRECTORY)
+        os.fsync(dir_fileno)
+        os.close(dir_fileno)
 
     def get_contact_file(self, reg):
         """Return name of contact file."""
@@ -457,20 +466,20 @@ To start a new run, stop the old one first with one or more of these:
         if not os.path.isfile(os.path.join(source, self.FILE_BASE_SUITE_RC)):
             raise SuiteServiceFileError("ERROR: no suite.rc in %s" % source)
 
-        # Suite service directory.
-        srv_d = self.get_suite_srv_dir(reg)
-
-        # Does symlink to suite source already exist?
-        target = os.path.join(srv_d, self.FILE_BASE_SOURCE)
-        try:
-            orig_source = os.readlink(target)
-        except OSError:
-            orig_source = None
-
         # Create service dir if necessary.
+        srv_d = self.get_suite_srv_dir(reg)
         mkdir_p(srv_d)
 
-        # Redirect an existing name to another suite?
+        # See if suite already has a source or not
+        try:
+            orig_source = os.readlink(
+                os.path.join(srv_d, self.FILE_BASE_SOURCE))
+        except OSError:
+            orig_source = None
+        else:
+            if not os.path.isabs(orig_source):
+                orig_source = os.path.normpath(
+                    os.path.join(srv_d, orig_source))
         if orig_source is not None and source != orig_source:
             if not redirect:
                 raise SuiteServiceFileError(
@@ -483,11 +492,19 @@ To start a new run, stop the old one first with one or more of these:
                 " directory will be overwritten.\n",
                 {'reg': reg, 'old': orig_source, 'new': source})
             # Remove symlink to the original suite.
-            os.unlink(target)
+            os.unlink(os.path.join(srv_d, self.FILE_BASE_SOURCE))
 
         # Create symlink to the suite, if it doesn't already exist.
-        if source != orig_source:
-            os.symlink(source, target)
+        if orig_source is None or source != orig_source:
+            target = os.path.join(srv_d, self.FILE_BASE_SOURCE)
+            if (os.path.abspath(source) ==
+                    os.path.abspath(os.path.dirname(srv_d))):
+                # If source happens to be the run directory,
+                # create .service/source -> ..
+                source_str = ".."
+            else:
+                source_str = source
+            os.symlink(source_str, target)
 
         print('REGISTERED %s -> %s' % (reg, source))
         return reg
