@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 """Task events manager.
 
 This module provides logic to:
-* Manage task messages (incoming or internal).
+* Manage task messages (internal, polled or received).
 * Set up retries on task job failures (submission or execution).
 * Generate task event handlers.
   * Retrieval of log files for completed remote jobs.
@@ -46,7 +46,7 @@ from cylc.task_job_logs import (
 from cylc.task_message import (
     ABORT_MESSAGE_PREFIX, FAIL_MESSAGE_PREFIX, VACATION_MESSAGE_PREFIX)
 from cylc.task_state import (
-    TASK_STATUSES_ACTIVE,
+    TASK_STATUSES_ACTIVE, TASK_STATUS_HELD,
     TASK_STATUS_READY, TASK_STATUS_SUBMITTED, TASK_STATUS_SUBMIT_RETRYING,
     TASK_STATUS_SUBMIT_FAILED, TASK_STATUS_RUNNING, TASK_STATUS_RETRYING,
     TASK_STATUS_FAILED, TASK_STATUS_SUCCEEDED)
@@ -99,7 +99,7 @@ class TaskEventsManager(object):
     """Task events manager.
 
     This class does the following:
-    * Manage task messages (incoming or otherwise).
+    * Manage task messages (received or otherwise).
     * Set up task (submission) retries on job (submission) failures.
     * Generate and manage task event handlers.
     """
@@ -115,8 +115,10 @@ class TaskEventsManager(object):
     HANDLER_MAIL = "event-mail"
     JOB_FAILED = "job failed"
     HANDLER_JOB_LOGS_RETRIEVE = "job-logs-retrieve"
-    IGNORED_INCOMING_FLAG = "(>ignored)"
-    INCOMING_FLAG = ">"
+    FLAG_INTERNAL = "(internal)"
+    FLAG_RECEIVED = "(received)"
+    FLAG_RECEIVED_IGNORED = "(received-ignored)"
+    FLAG_POLLED = "(polled)"
     KEY_EXECUTE_TIME_LIMIT = 'execution_time_limit'
     LEVELS = {
         "INFO": INFO,
@@ -127,12 +129,12 @@ class TaskEventsManager(object):
         "DEBUG": DEBUG,
     }
     NON_UNIQUE_EVENTS = ('warning', 'critical', 'custom')
-    POLLED_FLAG = "(polled)"
 
     def __init__(self, suite, proc_pool, suite_db_mgr, broadcast_mgr):
         self.suite = suite
         self.suite_url = None
         self.suite_cfg = {}
+        self.uuid_str = None
         self.proc_pool = proc_pool
         self.suite_db_mgr = suite_db_mgr
         self.broadcast_mgr = broadcast_mgr
@@ -278,21 +280,27 @@ class TaskEventsManager(object):
                 self._process_job_logs_retrieval(schd_ctx, ctx, id_keys)
 
     def process_message(
-            self, itask, severity, message, event_time=None, flag='',
-            submit_num=None):
-        """Parse an incoming task message and update task state.
+        self,
+        itask,
+        severity,
+        message,
+        event_time=None,
+        flag=FLAG_INTERNAL,
+        submit_num=None,
+    ):
+        """Parse an task message and update task state.
 
         Incoming, e.g. "succeeded at <TIME>", may be from task job or polling.
 
-        It is possible for my current state to be inconsistent with an incoming
-        message (whether normal or polled) e.g. due to a late poll result, or a
-        network outage, or manual state reset. To handle this, if a message
-        would take the task state backward, issue a poll to confirm instead of
-        changing state - then always believe the next message. Note that the
-        next message might not be the result of this confirmation poll, in the
-        unlikely event that a job emits a succession of messages very quickly,
-        but this is the best we can do without somehow uniquely associating
-        each poll with its result message.
+        It is possible for the current state of a task to be inconsistent with
+        a message (whether internal, received or polled) e.g. due to a late
+        poll result, or a network outage, or manual state reset. To handle
+        this, if a message would take the task state backward, issue a poll to
+        confirm instead of changing state - then always believe the next
+        message. Note that the next message might not be the result of this
+        confirmation poll, in the unlikely event that a job emits a succession
+        of messages very quickly, but this is the best we can do without
+        somehow uniquely associating each poll with its result message.
 
         Arguments:
             itask (cylc.task_proxy.TaskProxy):
@@ -305,10 +313,14 @@ class TaskEventsManager(object):
                 Event time stamp. Expect ISO8601 date time string.
                 If not specified, use current time.
             flag (str):
-                If specified, can be INCOMING_FLAG to indicate an incoming
-                message, POLLED_FLAG to indicate a message resulted from a
-                poll. Otherwise, the message is assumed to be generated by the
-                logic in the suite server program.
+                If specified, can be:
+                    FLAG_INTERNAL (default):
+                        To indicate an internal message.
+                    FLAG_RECEIVED:
+                        To indicate a message received from a job or an
+                        external source.
+                    FLAG_POLLED:
+                        To indicate a message resulted from a poll.
             submit_num (int):
                 The submit number of the task relevant for the message.
                 If not specified, use latest submit number.
@@ -318,26 +330,25 @@ class TaskEventsManager(object):
             True: if polling is required to confirm a reversal of status.
 
         """
-        # Log incoming messages
+        # Log messages
         if event_time is None:
             event_time = get_current_time_string()
         if submit_num is None:
             submit_num = itask.submit_num
-        if flag == self.INCOMING_FLAG and submit_num != itask.submit_num:
-            flag = self.IGNORED_INCOMING_FLAG
+        logfmt = r'[%s] status=%s: %s%s at %s for job(%02d)'
+        if flag == self.FLAG_RECEIVED and submit_num != itask.submit_num:
+            LOG.warning(
+                logfmt + r' != current job(%02d)',
+                itask, itask.state, self.FLAG_RECEIVED_IGNORED, message,
+                event_time, submit_num, itask.submit_num)
+            return
         LOG.log(
             self.LEVELS.get(severity, INFO),
-            r'[%s] -(current:%s)%s %s at %s',
-            itask, itask.state.status, flag, message, event_time)
-        if flag == self.IGNORED_INCOMING_FLAG:
-            LOG.warning(
-                '[%s] -submit-num=%02d: ignore message from job(%02d)',
-                itask, itask.submit_num, submit_num)
-            return
+            logfmt, itask, itask.state, flag, message, event_time, submit_num)
 
         # always update the suite state summary for latest message
-        if flag == self.POLLED_FLAG:
-            itask.set_summary_message('%s %s' % (message, self.POLLED_FLAG))
+        if flag == self.FLAG_POLLED:
+            itask.set_summary_message('%s %s' % (message, self.FLAG_POLLED))
         else:
             itask.set_summary_message(message)
 
@@ -346,30 +357,30 @@ class TaskEventsManager(object):
             message=message, is_completed=True)
 
         if message == TASK_OUTPUT_STARTED:
-            if (flag == self.INCOMING_FLAG
+            if (flag == self.FLAG_RECEIVED
                     and itask.state.is_gt(TASK_STATUS_RUNNING)):
                 return True
             self._process_message_started(itask, event_time)
         elif message == TASK_OUTPUT_SUCCEEDED:
             self._process_message_succeeded(itask, event_time)
         elif message == TASK_OUTPUT_FAILED:
-            if (flag == self.INCOMING_FLAG
+            if (flag == self.FLAG_RECEIVED
                     and itask.state.is_gt(TASK_STATUS_FAILED)):
                 return True
             self._process_message_failed(itask, event_time, self.JOB_FAILED)
         elif message == self.EVENT_SUBMIT_FAILED:
-            if (flag == self.INCOMING_FLAG
+            if (flag == self.FLAG_RECEIVED
                     and itask.state.is_gt(TASK_STATUS_SUBMIT_FAILED)):
                 return True
             self._process_message_submit_failed(itask, event_time)
         elif message == TASK_OUTPUT_SUBMITTED:
-            if (flag == self.INCOMING_FLAG
+            if (flag == self.FLAG_RECEIVED
                     and itask.state.is_gt(TASK_STATUS_SUBMITTED)):
                 return True
             self._process_message_submitted(itask, event_time)
         elif message.startswith(FAIL_MESSAGE_PREFIX):
             # Task received signal.
-            if (flag == self.INCOMING_FLAG
+            if (flag == self.FLAG_RECEIVED
                     and itask.state.is_gt(TASK_STATUS_FAILED)):
                 return True
             signal = message[len(FAIL_MESSAGE_PREFIX):]
@@ -379,7 +390,7 @@ class TaskEventsManager(object):
             self._process_message_failed(itask, event_time, self.JOB_FAILED)
         elif message.startswith(ABORT_MESSAGE_PREFIX):
             # Task aborted with message
-            if (flag == self.INCOMING_FLAG
+            if (flag == self.FLAG_RECEIVED
                     and itask.state.is_gt(TASK_STATUS_FAILED)):
                 return True
             aborted_with = message[len(ABORT_MESSAGE_PREFIX):]
@@ -416,7 +427,7 @@ class TaskEventsManager(object):
             # Note that all messages are logged already at the top.
             # No state change.
             LOG.debug(
-                '[%s] -(current:%s) unhandled: %s',
+                '[%s] status=%s: unhandled: %s',
                 itask, itask.state.status, message)
             if severity in [CRITICAL, ERROR, WARNING, INFO, DEBUG]:
                 severity = getLevelName(severity)
@@ -577,6 +588,10 @@ class TaskEventsManager(object):
 
     def _job_logs_retrieval_callback(self, proc_ctx, schd_ctx):
         """Call back when log job retrieval completes."""
+        if proc_ctx.ret_code:
+            LOG.error(proc_ctx)
+        else:
+            LOG.debug(proc_ctx)
         for id_key in proc_ctx.cmd_kwargs["id_keys"]:
             key1, point, name, submit_num = id_key
             try:
@@ -625,16 +640,19 @@ class TaskEventsManager(object):
                 self.setup_event_handlers(itask, "failed", message)
             LOG.critical(
                 "[%s] -job(%02d) %s", itask, itask.submit_num, "failed")
-        else:
-            # There is a retry lined up
+        elif itask.state.reset_state(
+            TASK_STATUS_RETRYING,
+            respect_hold_swap=True,
+        ):
             delay_msg = "retrying in %s" % (
                 itask.try_timers[TASK_STATUS_RETRYING].delay_timeout_as_str())
+            if itask.state.status == TASK_STATUS_HELD:
+                delay_msg = "%s (%s)" % (TASK_STATUS_HELD, delay_msg)
             msg = "failed, %s" % (delay_msg)
             LOG.info("[%s] -job(%02d) %s", itask, itask.submit_num, msg)
             itask.set_summary_message(msg)
-            if itask.state.reset_state(TASK_STATUS_RETRYING):
-                self.setup_event_handlers(
-                    itask, "retry", "%s, %s" % (self.JOB_FAILED, delay_msg))
+            self.setup_event_handlers(
+                itask, "retry", "%s, %s" % (self.JOB_FAILED, delay_msg))
         self._reset_job_timers(itask)
 
     def _process_message_started(self, itask, event_time):
@@ -688,7 +706,7 @@ class TaskEventsManager(object):
         if event_time is None:
             event_time = get_current_time_string()
         self.suite_db_mgr.put_update_task_jobs(itask, {
-            "time_submit_exit": get_current_time_string(),
+            "time_submit_exit": event_time,
             "submit_status": 1,
         })
         itask.summary['submit_method_id'] = None
@@ -701,17 +719,21 @@ class TaskEventsManager(object):
                 self.setup_event_handlers(
                     itask, self.EVENT_SUBMIT_FAILED,
                     'job %s' % self.EVENT_SUBMIT_FAILED)
-        else:
+        elif itask.state.reset_state(
+            TASK_STATUS_SUBMIT_RETRYING,
+            respect_hold_swap=True,
+        ):
             # There is a submission retry lined up.
             timer = itask.try_timers[TASK_STATUS_SUBMIT_RETRYING]
             delay_msg = "submit-retrying in %s" % timer.delay_timeout_as_str()
+            if itask.state.status == TASK_STATUS_HELD:
+                delay_msg = "%s (%s)" % (TASK_STATUS_HELD, delay_msg)
             msg = "%s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg)
             LOG.info("[%s] -job(%02d) %s", itask, itask.submit_num, msg)
             itask.set_summary_message(msg)
-            if itask.state.reset_state(TASK_STATUS_SUBMIT_RETRYING):
-                self.setup_event_handlers(
-                    itask, self.EVENT_SUBMIT_RETRY,
-                    "job %s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg))
+            self.setup_event_handlers(
+                itask, self.EVENT_SUBMIT_RETRY,
+                "job %s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg))
         self._reset_job_timers(itask)
 
     def _process_message_submitted(self, itask, event_time):
@@ -855,6 +877,7 @@ class TaskEventsManager(object):
                 handler_data = {
                     "event": quote(event),
                     "suite": quote(self.suite),
+                    'suite_uuid': quote(str(self.uuid_str)),
                     "point": quote(str(itask.point)),
                     "name": quote(itask.tdef.name),
                     "submit_num": itask.submit_num,
