@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ from cylc.task_job_logs import (
     JOB_LOG_JOB, get_task_job_log, get_task_job_job_log,
     get_task_job_activity_log, get_task_job_id, NN)
 from cylc.mkdir_p import mkdir_p
-from cylc.subprocpool import SuiteProcPool
+from cylc.subprocpool import SubProcPool
 from cylc.subprocctx import SubProcContext
 from cylc.task_action_timer import TaskActionTimer
 from cylc.task_events_mgr import TaskEventsManager, log_task_job_activity
@@ -75,7 +75,7 @@ class TaskJobManager(object):
 
     JOBS_KILL = 'jobs-kill'
     JOBS_POLL = 'jobs-poll'
-    JOBS_SUBMIT = SuiteProcPool.JOBS_SUBMIT
+    JOBS_SUBMIT = SubProcPool.JOBS_SUBMIT
     POLL_FAIL = 'poll failed'
     REMOTE_SELECT_MSG = 'waiting for remote host selection'
     REMOTE_INIT_MSG = 'remote host initialising'
@@ -239,7 +239,7 @@ class TaskJobManager(object):
             for itask in itasks:
                 # Log and persist
                 LOG.info(
-                    '[%s] -submit-num=%d, owner@host=%s',
+                    '[%s] -submit-num=%02d, owner@host=%s',
                     itask, itask.submit_num, owner_at_host)
                 self.suite_db_mgr.put_insert_task_jobs(itask, {
                     'is_manual_submit': itask.is_manual_submit,
@@ -297,11 +297,11 @@ class TaskJobManager(object):
                 '%s ... # will invoke in batches, sizes=%s',
                 cmd, [len(b) for b in itasks_batches])
             for i, itasks_batch in enumerate(itasks_batches):
-                stdin_file_paths = []
+                stdin_files = []
                 job_log_dirs = []
                 for itask in itasks_batch:
                     if remote_mode:
-                        stdin_file_paths.append(
+                        stdin_files.append(
                             get_task_job_job_log(
                                 suite, itask.point, itask.tdef.name,
                                 itask.submit_num))
@@ -318,7 +318,7 @@ class TaskJobManager(object):
                     SubProcContext(
                         self.JOBS_SUBMIT,
                         cmd + job_log_dirs,
-                        stdin_file_paths=stdin_file_paths,
+                        stdin_files=stdin_files,
                         job_log_dirs=job_log_dirs,
                         **kwargs
                     ),
@@ -426,7 +426,7 @@ class TaskJobManager(object):
                 handle.write(owner_at_host + line)
         except IOError as exc:
             LOG.warning("%s: write failed\n%s" % (job_activity_log, exc))
-            LOG.warning("[%s] -%s%s", itask, owner_at_host, + line)
+            LOG.warning("[%s] -%s%s", itask, owner_at_host, line)
 
     def _kill_task_jobs_callback(self, ctx, suite, itasks):
         """Callback when kill tasks command exits."""
@@ -497,14 +497,7 @@ class TaskJobManager(object):
         out = ctx.out
         if not out:
             out = ""
-            # Something is very wrong here
-            # Fallback to use "job_log_dirs" list to report the problem
-            job_log_dirs = ctx.cmd_kwargs.get("job_log_dirs", [])
-            for job_log_dir in job_log_dirs:
-                point, name, submit_num = job_log_dir.split(os.sep, 2)
-                itask = tasks[(point, name, submit_num)]
-                out += (self.batch_sys_mgr.OUT_PREFIX_SUMMARY +
-                        "|".join([ctx.timestamp, job_log_dir, "1"]) + "\n")
+        bad_tasks = dict(tasks)
         for line in out.splitlines(True):
             for prefix, callback in handlers:
                 if line.startswith(prefix):
@@ -512,12 +505,20 @@ class TaskJobManager(object):
                     try:
                         path = line.split("|", 2)[1]  # timestamp, path, status
                         point, name, submit_num = path.split(os.sep, 2)
+                        if prefix == self.batch_sys_mgr.OUT_PREFIX_SUMMARY:
+                            del bad_tasks[(point, name, submit_num)]
                         itask = tasks[(point, name, submit_num)]
                         callback(suite, itask, ctx, line)
-                    except (LookupError, ValueError) as exc:
+                    except (LookupError, ValueError, KeyError) as exc:
                         LOG.warning(
                             'Unhandled %s output: %s', ctx.cmd_key, line)
                         LOG.exception(exc)
+        # Task jobs that are in the original command but did not get a status
+        # in the output. Handle as failures.
+        for key, itask in sorted(bad_tasks.items()):
+            line = (
+                "|".join([ctx.timestamp, os.sep.join(key), "1"]) + "\n")
+            summary_callback(suite, itask, ctx, line)
 
     def _poll_task_jobs_callback(self, ctx, suite, itasks):
         """Callback when poll tasks command exits."""
@@ -537,8 +538,13 @@ class TaskJobManager(object):
 
         # See cylc.batch_sys_manager.JobPollContext
         try:
-            job_log_dir, context = line.split('|')[1:4]
+            job_log_dir, context = line.split('|')[1:3]
             items = json.loads(context)
+            jp_ctx = JobPollContext(job_log_dir, **items)
+        except TypeError:
+            itask.set_summary_message(self.POLL_FAIL)
+            ctx.cmd = cmd_ctx.cmd  # print original command on failure
+            return
         except ValueError:
             # back compat for cylc 7.7.1 and previous
             try:
@@ -554,9 +560,7 @@ class TaskJobManager(object):
         finally:
             log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
 
-        jp_ctx = JobPollContext(job_log_dir, **items)
-
-        flag = self.task_events_mgr.POLLED_FLAG
+        flag = self.task_events_mgr.FLAG_POLLED
         if jp_ctx.run_status == 1 and jp_ctx.run_signal in ["ERR", "EXIT"]:
             # Failed normally
             self.task_events_mgr.process_message(
@@ -613,7 +617,7 @@ class TaskJobManager(object):
             ctx.ret_code = 0
             self.task_events_mgr.process_message(
                 itask, severity, message, event_time,
-                self.task_events_mgr.POLLED_FLAG)
+                self.task_events_mgr.FLAG_POLLED)
         log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
 
     def _run_job_cmd(self, cmd_key, suite, itasks, callback):
@@ -710,7 +714,7 @@ class TaskJobManager(object):
                 ctx.cmd = cmd_ctx.cmd  # print original command on failure
         log_task_job_activity(ctx, suite, itask.point, itask.tdef.name)
 
-        if ctx.ret_code == SuiteProcPool.RET_CODE_SUITE_STOPPING:
+        if ctx.ret_code == SubProcPool.RET_CODE_SUITE_STOPPING:
             return
 
         try:
