@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
+# Copyright (C) 2008-2019 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ from glob import glob
 import os
 import sys
 from jinja2 import (
+    select_autoescape,
     BaseLoader,
     ChoiceLoader,
     Environment,
@@ -93,9 +94,14 @@ def jinja2environment(dir_=None):
         dir_ = os.getcwd()
 
     env = Environment(
+        autoescape=select_autoescape(
+            enabled_extensions=(),
+            default_for_string=False,
+            default=False),
         loader=ChoiceLoader([FileSystemLoader(dir_), PyModuleLoader()]),
         undefined=StrictUndefined,
-        extensions=['jinja2.ext.do'])
+        extensions=['jinja2.ext.do']
+    )
 
     # Load any custom Jinja2 filters, tests or globals in the suite
     # definition directory
@@ -111,7 +117,7 @@ def jinja2environment(dir_=None):
                 os.path.join(dir_, nspdir),
                 os.path.join(os.environ['HOME'], '.cylc', nspdir)]:
             if os.path.isdir(fdir):
-                sys.path.append(os.path.abspath(fdir))
+                sys.path.insert(1, os.path.abspath(fdir))
                 for name in glob(os.path.join(fdir, '*.py')):
                     fname = os.path.splitext(os.path.basename(name))[0]
                     # TODO - EXCEPTION HANDLING FOR LOADING CUSTOM FILTERS
